
export * from "./circuit";
export * from "./@types/lit-listener-sdk";

/// <reference types="node" />
export declare class Circuit extends EventEmitter {
    private isRunning;
    private useSecureKey;
    private conditions;
    private pendingBroadcasts;
    private monitor;
    private conditionalLogic?;
    private satisfiedConditions;
    private actions;
    private conditionExecutedCount;
    private litActionCompletionCount;
    private conditionMonitorExecutions?;
    private startDate?;
    private endDate?;
    private maxLitActionCompletions?;
    private logSize;
    private logs;
    private logIndex;
    private signer;
    private litClient;
    private pkpContract;
    private pkpHelperContract;
    private code;
    private secureKey;
    private ipfsCID;
    private publicKey;
    private pkpAddress;
    private authSig;
    private hasSetActionHelperFunction;
    private continueRun;
    private actionFunctions;
    private emitter;
    private jsParameters;
    private errorHandlingModeStrict;
    private lastSuccessfulNonce;
    private sessionSig;
    constructor(signer?: ethers.Signer, pkpContractAddress?: string, pkpHelperContractAddress?: string, errorHandlingModeStrict?: boolean);
    setConditions: (conditions: Condition[]) => void;
    setConditionalLogic: (logic: IConditionalLogic) => void;
    executionConstraints: (options: IExecutionConstraints) => void;
    setActions: (actions: Action[], useSecureKey?: boolean) => Promise<{
        unsignedTransactionDataObject: {
            [key: string]: LitUnsignedTransaction;
        };
        litActionCode: string;
        secureKey: string;
    }>;
    generateUnsignedTransactionData: (data: UnsignedTransactionData, providerURL: string) => Promise<LitUnsignedTransaction>;
    getIPFSHash: (code: string) => Promise<string>;
    mintGrantBurnPKP: (cidIPFS: string) => Promise<{
        tokenId: string;
        publicKey: `0x04${string}` | string;
        address: string;
    }>;
    start: ({ publicKey, ipfsCID, sessionSig, secureKey, broadcast, }: {
        publicKey: `0x04${string}` | string;
        ipfsCID?: string;
        sessionSig?: SessionSigsMap;
        secureKey?: string;
        broadcast?: boolean;
    }) => Promise<void>;
    getLogs: (category?: LogCategory) => ILogEntry[];
    generateSessionSignature: (pkpPublicKey: string, resources?: LitResourceAbilityRequest[], chainId?: number, uri?: string, version?: string) => Promise<SessionSigsMap>;
    interrupt: () => void;
    private mintNextPKP;
    private getPubKeyByPKPTokenId;
    private runLitAction;
    private connectLit;
    private checkExecutionLimitations;
    private broadcastContractActions;
    private checkConditionalLogicAndRun;
    private log;
}

export declare class ConditionMonitor extends EventEmitter {
    constructor();
    createCondition: (condition: WebhookCondition | ContractCondition, errorHandlingModeStrict: boolean) => Promise<void>;
    private startMonitoringWebHook;
    private startMonitoringContract;
    private checkAgainstExpected;
    private compareValues;
    private deepEqualObjects;
    private checkProvider;
    private retry;
}

export declare enum LitChainIds {
    ethereum = 1,
    polygon = 137,
    fantom = 250,
    bsc = 56,
    arbitrum = 42161,
    avalanche = 43114,
    fuji = 43113,
    harmony = 1666600000,
    mumbai = 80001,
    goerli = 5,
    cronos = 25,
    optimism = 10,
    celo = 42220,
    aurora = 1313161554,
    alfajores = 44787,
    xdc = 50,
    evmos = 9001,
    evmosTestnet = 9000,
    hardhat = 31337
}
export declare enum CHAIN_NAME {
    ETHEREUM = "ethereum",
    POLYGON = "polygon",
    FANTOM = "fantom",
    BSC = "bsc",
    ARBITRUM = "arbitrum",
    AVALANCHE = "avalanche",
    FUJI = "fuji",
    HARMONY = "harmony",
    MUMBAI = "mumbai",
    GOERLI = "goerli",
    CRONOS = "cronos",
    OPTIMISM = "optimism",
    CELO = "celo",
    AURORA = "aurora",
    ALFAJORES = "alfajores",
    XDC = "xdc",
    EVMOS = "evmos",
    EVMOSTESTNET = "evmosTestnet",
    HARDHAT = "hardhat"
}
export declare enum RunStatus {
    EXIT_RUN = 0,
    ACTION_RUN = 1,
    CONTINUE_RUN = 2
}
export interface LitActionsSDK {
    signEcdsa: ({ toSign, publicKey, sigName, }: {
        toSign: Uint8Array;
        publicKey: `0x04${string}` | string;
        sigName: string;
    }) => Promise<void>;
    getLatestNonce: (chain: string) => Promise<number>;
    setResponse: ({ response }: {
        response: string;
    }) => void;
}
export type UnsignedTransactionData = {
    contractAddress?: `0x${string}`;
    nonce?: number;
    gasLimit?: BigNumberish;
    value?: BigNumberish;
    chainId?: CHAIN_NAME;
    maxPriorityFeePerGas?: BigNumberish;
    maxFeePerGas?: BigNumberish;
    from: `0x${string}`;
    functionName: string;
    args: any[];
    abi: ethers.ContractInterface;
};
export type UnsignedTransaction = {
    to?: `0x${string}`;
    nonce?: number;
    gasLimit?: BigNumberish;
    data?: BytesLike;
    value?: BigNumberish;
    chainId?: number;
    type?: number | null;
    accessList?: AccessListish;
    maxPriorityFeePerGas?: BigNumberish;
    maxFeePerGas?: BigNumberish;
};
export type LitUnsignedTransaction = UnsignedTransaction & {
    from: `0x${string}` | "{{publicKey}}";
};
export interface CustomAction {
    type: "custom";
    priority: number;
    code: string;
    args?: Object;
}
export interface ContractAction {
    type: "contract";
    priority: number;
    contractAddress: `0x${string}`;
    abi: ethers.ContractInterface;
    functionName: string;
    chainId: CHAIN_NAME;
    providerURL: string;
    nonce?: number;
    gasLimit?: BigNumberish;
    value?: BigNumberish;
    from?: `0x${string}`;
    maxPriorityFeePerGas?: BigNumberish;
    maxFeePerGas?: BigNumberish;
    args?: any[];
}
export interface FetchAction {
    type: "fetch";
    priority: number;
    baseUrl: string;
    endpoint: string;
    responsePath: string;
    apiKey?: string;
    toSign?: Uint8Array;
    signCondition?: {
        type: "&&" | "||";
        operator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=";
        value: number | string | bigint | string[] | number[] | bigint[] | undefined | (string | number | bigint)[];
    }[];
}
export type Action = CustomAction | ContractAction | FetchAction;
export interface IThresholdConditionalLogic {
    type: "THRESHOLD";
    value: number;
    interval?: number;
}
export interface ITargetConditionalLogic {
    type: "TARGET";
    targetCondition: string;
    interval?: number;
}
export interface IEveryConditionalLogic {
    type: "EVERY";
    interval?: number;
}
export type IConditionalLogic = IThresholdConditionalLogic | ITargetConditionalLogic | IEveryConditionalLogic;
export interface IContractCondition {
    contractAddress: `0x${string}`;
    abi: ethers.ContractInterface;
    chainId: CHAIN_NAME;
    providerURL: string;
    eventName: string;
    eventArgName: string[];
    expectedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[];
    matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=";
    onMatched: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onUnMatched: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onError: (error: Error) => void;
}
export declare class ContractCondition implements IContractCondition {
    contractAddress: `0x${string}`;
    abi: ethers.ContractInterface;
    chainId: CHAIN_NAME;
    providerURL: string;
    eventName: string;
    eventArgName: string[];
    expectedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[];
    matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=";
    onMatched: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onUnMatched: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onError: (error: Error) => void;
    id?: string;
    sdkOnMatched?: () => Promise<void>;
    sdkOnUnMatched?: () => Promise<void>;
    constructor(contractAddress: `0x${string}`, abi: ethers.ContractInterface, chainId: CHAIN_NAME, providerURL: string, eventName: string, eventArgName: string[], expectedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[], matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=", onMatched?: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>, onUnMatched?: (emittedValue: number[] | string[] | bigint[] | object[] | (string | number | bigint | object)[]) => Promise<void>, onError?: (error: Error) => void);
}
export interface IWebhookCondition {
    baseUrl: string;
    endpoint: string;
    responsePath: string;
    expectedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[];
    matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=";
    apiKey?: string;
    onMatched: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onUnMatched: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onError: (error: Error) => void;
}
export declare class WebhookCondition implements IWebhookCondition {
    baseUrl: string;
    endpoint: string;
    responsePath: string;
    expectedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[];
    matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=";
    apiKey?: string;
    onMatched: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onUnMatched: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>;
    onError: (error: Error) => void;
    id?: string;
    sdkOnMatched?: () => Promise<void>;
    sdkOnUnMatched?: () => Promise<void>;
    constructor(baseUrl: string, endpoint: string, responsePath: string, expectedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[], matchOperator: "<" | ">" | "==" | "===" | "!==" | "!=" | ">=" | "<=", apiKey?: string, onMatched?: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>, onUnMatched?: (emittedValue: number | string | number[] | string[] | bigint | bigint[] | object | object[] | (string | number | bigint | object)[]) => Promise<void>, onError?: (error: Error) => void);
}
export type Condition = ContractCondition | WebhookCondition;
export interface IExecutionConstraints {
    conditionMonitorExecutions?: number;
    startDate?: Date;
    endDate?: Date;
    maxLitActionCompletions?: number;
}
export declare enum LogCategory {
    ERROR = 0,
    RESPONSE = 1,
    CONDITION = 2,
    BROADCAST = 3,
    EXECUTION = 4
}
export interface ILogEntry {
    category: LogCategory;
    message: string;
    responseObject: string;
    isoDate: string;
}

export declare const PKP_CONTRACT_ADDRESS = "0x58582b93d978F30b4c4E812A16a7b31C035A69f7";
export declare const PKP_PERMISSIONS_CONTRACT_ADDRESS = "0xD01c9C30f8F6fa443721629775e1CC7DD9c9e209";
export declare const PKP_HELPER_CONTRACT_ADDRESS = "0xE4DEe6d1DCe79CB752d1641dC055F197eb674DcB";
export declare const CHRONICLE_PROVIDER = "https://chain-rpc.litprotocol.com/http";
export declare const DENO_BUNDLED = "";

export declare const generateSessionSig: (client: LitNodeClient, signer: ethers.Signer, pkpPublicKey: string, resources?: LitResourceAbilityRequest[], chainId?: number, uri?: string, version?: string) => Promise<SessionSigsMap>;
export declare const getBytesFromMultihash: (multihash: string) => string;
export declare const generateSecureRandomKey: () => string;
export declare const hashHex: (input: string) => string;
export declare const bundleCodeManual: (dynamicCode: string) => string;







export declare namespace IDiamond {
    type FacetCutStruct = {
        facetAddress: PromiseOrValue<string>;
        action: PromiseOrValue<BigNumberish>;
        functionSelectors: PromiseOrValue<BytesLike>[];
    };
    type FacetCutStructOutput = [string, number, string[]] & {
        facetAddress: string;
        action: number;
        functionSelectors: string[];
    };
}
export declare namespace IDiamondLoupe {
    type FacetStruct = {
        facetAddress: PromiseOrValue<string>;
        functionSelectors: PromiseOrValue<BytesLike>[];
    };
    type FacetStructOutput = [string, string[]] & {
        facetAddress: string;
        functionSelectors: string[];
    };
}
export declare namespace IPubkeyRouter {
    type SignatureStruct = {
        r: PromiseOrValue<BytesLike>;
        s: PromiseOrValue<BytesLike>;
        v: PromiseOrValue<BigNumberish>;
    };
    type SignatureStructOutput = [string, string, number] & {
        r: string;
        s: string;
        v: number;
    };
}
export interface PKPNFTInterface extends utils.Interface {
    functions: {
        "diamondCut((address,uint8,bytes4[])[],address,bytes)": FunctionFragment;
        "facetAddress(bytes4)": FunctionFragment;
        "facetAddresses()": FunctionFragment;
        "facetFunctionSelectors(address)": FunctionFragment;
        "facets()": FunctionFragment;
        "owner()": FunctionFragment;
        "transferOwnership(address)": FunctionFragment;
        "approve(address,uint256)": FunctionFragment;
        "balanceOf(address)": FunctionFragment;
        "burn(uint256)": FunctionFragment;
        "claimAndMint(uint256,bytes32,(bytes32,bytes32,uint8)[])": FunctionFragment;
        "exists(uint256)": FunctionFragment;
        "freeMintSigner()": FunctionFragment;
        "getApproved(uint256)": FunctionFragment;
        "getEthAddress(uint256)": FunctionFragment;
        "getNextDerivedKeyId()": FunctionFragment;
        "getPkpNftMetadataAddress()": FunctionFragment;
        "getPkpPermissionsAddress()": FunctionFragment;
        "getPubkey(uint256)": FunctionFragment;
        "getRouterAddress()": FunctionFragment;
        "getStakingAddress()": FunctionFragment;
        "initialize()": FunctionFragment;
        "isApprovedForAll(address,address)": FunctionFragment;
        "mintCost()": FunctionFragment;
        "mintGrantAndBurnNext(uint256,bytes)": FunctionFragment;
        "mintNext(uint256)": FunctionFragment;
        "name()": FunctionFragment;
        "ownerOf(uint256)": FunctionFragment;
        "prefixed(bytes32)": FunctionFragment;
        "redeemedFreeMintIds(uint256)": FunctionFragment;
        "safeTransferFrom(address,address,uint256)": FunctionFragment;
        "safeTransferFrom(address,address,uint256,bytes)": FunctionFragment;
        "setApprovalForAll(address,bool)": FunctionFragment;
        "setContractResolver(address)": FunctionFragment;
        "setFreeMintSigner(address)": FunctionFragment;
        "setMintCost(uint256)": FunctionFragment;
        "supportsInterface(bytes4)": FunctionFragment;
        "symbol()": FunctionFragment;
        "tokenByIndex(uint256)": FunctionFragment;
        "tokenOfOwnerByIndex(address,uint256)": FunctionFragment;
        "tokenURI(uint256)": FunctionFragment;
        "totalSupply()": FunctionFragment;
        "transferFrom(address,address,uint256)": FunctionFragment;
        "withdraw()": FunctionFragment;
    };
    getFunction(nameOrSignatureOrTopic: "diamondCut" | "facetAddress" | "facetAddresses" | "facetFunctionSelectors" | "facets" | "owner" | "transferOwnership" | "approve" | "balanceOf" | "burn" | "claimAndMint" | "exists" | "freeMintSigner" | "getApproved" | "getEthAddress" | "getNextDerivedKeyId" | "getPkpNftMetadataAddress" | "getPkpPermissionsAddress" | "getPubkey" | "getRouterAddress" | "getStakingAddress" | "initialize" | "isApprovedForAll" | "mintCost" | "mintGrantAndBurnNext" | "mintNext" | "name" | "ownerOf" | "prefixed" | "redeemedFreeMintIds" | "safeTransferFrom(address,address,uint256)" | "safeTransferFrom(address,address,uint256,bytes)" | "setApprovalForAll" | "setContractResolver" | "setFreeMintSigner" | "setMintCost" | "supportsInterface" | "symbol" | "tokenByIndex" | "tokenOfOwnerByIndex" | "tokenURI" | "totalSupply" | "transferFrom" | "withdraw"): FunctionFragment;
    encodeFunctionData(functionFragment: "diamondCut", values: [
        IDiamond.FacetCutStruct[],
        PromiseOrValue<string>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "facetAddress", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "facetAddresses", values?: undefined): string;
    encodeFunctionData(functionFragment: "facetFunctionSelectors", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "facets", values?: undefined): string;
    encodeFunctionData(functionFragment: "owner", values?: undefined): string;
    encodeFunctionData(functionFragment: "transferOwnership", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "approve", values: [PromiseOrValue<string>, PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "balanceOf", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "burn", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "claimAndMint", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        IPubkeyRouter.SignatureStruct[]
    ]): string;
    encodeFunctionData(functionFragment: "exists", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "freeMintSigner", values?: undefined): string;
    encodeFunctionData(functionFragment: "getApproved", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getEthAddress", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getNextDerivedKeyId", values?: undefined): string;
    encodeFunctionData(functionFragment: "getPkpNftMetadataAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "getPkpPermissionsAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "getPubkey", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getRouterAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "getStakingAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "initialize", values?: undefined): string;
    encodeFunctionData(functionFragment: "isApprovedForAll", values: [PromiseOrValue<string>, PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "mintCost", values?: undefined): string;
    encodeFunctionData(functionFragment: "mintGrantAndBurnNext", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "mintNext", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "name", values?: undefined): string;
    encodeFunctionData(functionFragment: "ownerOf", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "prefixed", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "redeemedFreeMintIds", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "safeTransferFrom(address,address,uint256)", values: [
        PromiseOrValue<string>,
        PromiseOrValue<string>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "safeTransferFrom(address,address,uint256,bytes)", values: [
        PromiseOrValue<string>,
        PromiseOrValue<string>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "setApprovalForAll", values: [PromiseOrValue<string>, PromiseOrValue<boolean>]): string;
    encodeFunctionData(functionFragment: "setContractResolver", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "setFreeMintSigner", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "setMintCost", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "supportsInterface", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "symbol", values?: undefined): string;
    encodeFunctionData(functionFragment: "tokenByIndex", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "tokenOfOwnerByIndex", values: [PromiseOrValue<string>, PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "tokenURI", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "totalSupply", values?: undefined): string;
    encodeFunctionData(functionFragment: "transferFrom", values: [
        PromiseOrValue<string>,
        PromiseOrValue<string>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "withdraw", values?: undefined): string;
    decodeFunctionResult(functionFragment: "diamondCut", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetAddresses", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetFunctionSelectors", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facets", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "owner", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "transferOwnership", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "approve", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "balanceOf", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "burn", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "claimAndMint", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "exists", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "freeMintSigner", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getApproved", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getEthAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getNextDerivedKeyId", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPkpNftMetadataAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPkpPermissionsAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPubkey", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getRouterAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getStakingAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "initialize", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isApprovedForAll", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "mintCost", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "mintGrantAndBurnNext", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "mintNext", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "name", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "ownerOf", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "prefixed", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "redeemedFreeMintIds", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "safeTransferFrom(address,address,uint256)", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "safeTransferFrom(address,address,uint256,bytes)", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setApprovalForAll", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setContractResolver", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setFreeMintSigner", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setMintCost", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "supportsInterface", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "symbol", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "tokenByIndex", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "tokenOfOwnerByIndex", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "tokenURI", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "totalSupply", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "transferFrom", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "withdraw", data: BytesLike): Result;
    events: {
        "DiamondCut(tuple[],address,bytes)": EventFragment;
        "OwnershipTransferred(address,address)": EventFragment;
        "Approval(address,address,uint256)": EventFragment;
        "ApprovalForAll(address,address,bool)": EventFragment;
        "ContractResolverAddressSet(address)": EventFragment;
        "FreeMintSignerSet(address)": EventFragment;
        "Initialized(uint8)": EventFragment;
        "MintCostSet(uint256)": EventFragment;
        "PKPMinted(uint256,bytes)": EventFragment;
        "Transfer(address,address,uint256)": EventFragment;
        "Withdrew(uint256)": EventFragment;
    };
    getEvent(nameOrSignatureOrTopic: "DiamondCut"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "OwnershipTransferred"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "Approval"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "ApprovalForAll"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "ContractResolverAddressSet"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "FreeMintSignerSet"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "Initialized"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "MintCostSet"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "PKPMinted"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "Transfer"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "Withdrew"): EventFragment;
}
export interface DiamondCutEventObject {
    _diamondCut: IDiamond.FacetCutStructOutput[];
    _init: string;
    _calldata: string;
}
export type DiamondCutEvent = TypedEvent<[
    IDiamond.FacetCutStructOutput[],
    string,
    string
], DiamondCutEventObject>;
export type DiamondCutEventFilter = TypedEventFilter<DiamondCutEvent>;
export interface OwnershipTransferredEventObject {
    previousOwner: string;
    newOwner: string;
}
export type OwnershipTransferredEvent = TypedEvent<[
    string,
    string
], OwnershipTransferredEventObject>;
export type OwnershipTransferredEventFilter = TypedEventFilter<OwnershipTransferredEvent>;
export interface ApprovalEventObject {
    owner: string;
    approved: string;
    tokenId: BigNumber;
}
export type ApprovalEvent = TypedEvent<[
    string,
    string,
    BigNumber
], ApprovalEventObject>;
export type ApprovalEventFilter = TypedEventFilter<ApprovalEvent>;
export interface ApprovalForAllEventObject {
    owner: string;
    operator: string;
    approved: boolean;
}
export type ApprovalForAllEvent = TypedEvent<[
    string,
    string,
    boolean
], ApprovalForAllEventObject>;
export type ApprovalForAllEventFilter = TypedEventFilter<ApprovalForAllEvent>;
export interface ContractResolverAddressSetEventObject {
    newResolverAddress: string;
}
export type ContractResolverAddressSetEvent = TypedEvent<[
    string
], ContractResolverAddressSetEventObject>;
export type ContractResolverAddressSetEventFilter = TypedEventFilter<ContractResolverAddressSetEvent>;
export interface FreeMintSignerSetEventObject {
    newFreeMintSigner: string;
}
export type FreeMintSignerSetEvent = TypedEvent<[
    string
], FreeMintSignerSetEventObject>;
export type FreeMintSignerSetEventFilter = TypedEventFilter<FreeMintSignerSetEvent>;
export interface InitializedEventObject {
    version: number;
}
export type InitializedEvent = TypedEvent<[number], InitializedEventObject>;
export type InitializedEventFilter = TypedEventFilter<InitializedEvent>;
export interface MintCostSetEventObject {
    newMintCost: BigNumber;
}
export type MintCostSetEvent = TypedEvent<[BigNumber], MintCostSetEventObject>;
export type MintCostSetEventFilter = TypedEventFilter<MintCostSetEvent>;
export interface PKPMintedEventObject {
    tokenId: BigNumber;
    pubkey: string;
}
export type PKPMintedEvent = TypedEvent<[
    BigNumber,
    string
], PKPMintedEventObject>;
export type PKPMintedEventFilter = TypedEventFilter<PKPMintedEvent>;
export interface TransferEventObject {
    from: string;
    to: string;
    tokenId: BigNumber;
}
export type TransferEvent = TypedEvent<[
    string,
    string,
    BigNumber
], TransferEventObject>;
export type TransferEventFilter = TypedEventFilter<TransferEvent>;
export interface WithdrewEventObject {
    amount: BigNumber;
}
export type WithdrewEvent = TypedEvent<[BigNumber], WithdrewEventObject>;
export type WithdrewEventFilter = TypedEventFilter<WithdrewEvent>;
export interface PKPNFT extends BaseContract {
    connect(signerOrProvider: Signer | Provider | string): this;
    attach(addressOrName: string): this;
    deployed(): Promise<this>;
    interface: PKPNFTInterface;
    queryFilter<TEvent extends TypedEvent>(event: TypedEventFilter<TEvent>, fromBlockOrBlockhash?: string | number | undefined, toBlock?: string | number | undefined): Promise<Array<TEvent>>;
    listeners<TEvent extends TypedEvent>(eventFilter?: TypedEventFilter<TEvent>): Array<TypedListener<TEvent>>;
    listeners(eventName?: string): Array<Listener>;
    removeAllListeners<TEvent extends TypedEvent>(eventFilter: TypedEventFilter<TEvent>): this;
    removeAllListeners(eventName?: string): this;
    off: OnEvent<this>;
    on: OnEvent<this>;
    once: OnEvent<this>;
    removeListener: OnEvent<this>;
    functions: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[string] & {
            facetAddress_: string;
        }>;
        facetAddresses(overrides?: CallOverrides): Promise<[string[]] & {
            facetAddresses_: string[];
        }>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[string[]] & {
            _facetFunctionSelectors: string[];
        }>;
        facets(overrides?: CallOverrides): Promise<[
            IDiamondLoupe.FacetStructOutput[]
        ] & {
            facets_: IDiamondLoupe.FacetStructOutput[];
        }>;
        owner(overrides?: CallOverrides): Promise<[string] & {
            owner_: string;
        }>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        approve(to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        balanceOf(owner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[BigNumber]>;
        burn(tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        claimAndMint(keyType: PromiseOrValue<BigNumberish>, derivedKeyId: PromiseOrValue<BytesLike>, signatures: IPubkeyRouter.SignatureStruct[], overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        exists(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[boolean]>;
        freeMintSigner(overrides?: CallOverrides): Promise<[string]>;
        getApproved(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        getNextDerivedKeyId(overrides?: CallOverrides): Promise<[string]>;
        getPkpNftMetadataAddress(overrides?: CallOverrides): Promise<[string]>;
        getPkpPermissionsAddress(overrides?: CallOverrides): Promise<[string]>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        getRouterAddress(overrides?: CallOverrides): Promise<[string]>;
        getStakingAddress(overrides?: CallOverrides): Promise<[string]>;
        initialize(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        isApprovedForAll(owner: PromiseOrValue<string>, operator: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[boolean]>;
        mintCost(overrides?: CallOverrides): Promise<[BigNumber]>;
        mintGrantAndBurnNext(keyType: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        mintNext(keyType: PromiseOrValue<BigNumberish>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        name(overrides?: CallOverrides): Promise<[string]>;
        ownerOf(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        prefixed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[string]>;
        redeemedFreeMintIds(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[boolean]>;
        "safeTransferFrom(address,address,uint256)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        "safeTransferFrom(address,address,uint256,bytes)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, data: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setApprovalForAll(operator: PromiseOrValue<string>, approved: PromiseOrValue<boolean>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setFreeMintSigner(newFreeMintSigner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setMintCost(newMintCost: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        supportsInterface(interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        symbol(overrides?: CallOverrides): Promise<[string]>;
        tokenByIndex(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[BigNumber]>;
        tokenOfOwnerByIndex(owner: PromiseOrValue<string>, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[BigNumber]>;
        tokenURI(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        totalSupply(overrides?: CallOverrides): Promise<[BigNumber]>;
        transferFrom(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        withdraw(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
    };
    diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
    facetAddresses(overrides?: CallOverrides): Promise<string[]>;
    facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<string[]>;
    facets(overrides?: CallOverrides): Promise<IDiamondLoupe.FacetStructOutput[]>;
    owner(overrides?: CallOverrides): Promise<string>;
    transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    approve(to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    balanceOf(owner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
    burn(tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    claimAndMint(keyType: PromiseOrValue<BigNumberish>, derivedKeyId: PromiseOrValue<BytesLike>, signatures: IPubkeyRouter.SignatureStruct[], overrides?: PayableOverrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    exists(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
    freeMintSigner(overrides?: CallOverrides): Promise<string>;
    getApproved(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    getNextDerivedKeyId(overrides?: CallOverrides): Promise<string>;
    getPkpNftMetadataAddress(overrides?: CallOverrides): Promise<string>;
    getPkpPermissionsAddress(overrides?: CallOverrides): Promise<string>;
    getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    getRouterAddress(overrides?: CallOverrides): Promise<string>;
    getStakingAddress(overrides?: CallOverrides): Promise<string>;
    initialize(overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    isApprovedForAll(owner: PromiseOrValue<string>, operator: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
    mintCost(overrides?: CallOverrides): Promise<BigNumber>;
    mintGrantAndBurnNext(keyType: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: PayableOverrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    mintNext(keyType: PromiseOrValue<BigNumberish>, overrides?: PayableOverrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    name(overrides?: CallOverrides): Promise<string>;
    ownerOf(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    prefixed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
    redeemedFreeMintIds(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
    "safeTransferFrom(address,address,uint256)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    "safeTransferFrom(address,address,uint256,bytes)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, data: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setApprovalForAll(operator: PromiseOrValue<string>, approved: PromiseOrValue<boolean>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setFreeMintSigner(newFreeMintSigner: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setMintCost(newMintCost: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    supportsInterface(interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    symbol(overrides?: CallOverrides): Promise<string>;
    tokenByIndex(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
    tokenOfOwnerByIndex(owner: PromiseOrValue<string>, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
    tokenURI(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    totalSupply(overrides?: CallOverrides): Promise<BigNumber>;
    transferFrom(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    withdraw(overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    callStatic: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
        facetAddresses(overrides?: CallOverrides): Promise<string[]>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<string[]>;
        facets(overrides?: CallOverrides): Promise<IDiamondLoupe.FacetStructOutput[]>;
        owner(overrides?: CallOverrides): Promise<string>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        approve(to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        balanceOf(owner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        burn(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        claimAndMint(keyType: PromiseOrValue<BigNumberish>, derivedKeyId: PromiseOrValue<BytesLike>, signatures: IPubkeyRouter.SignatureStruct[], overrides?: CallOverrides): Promise<BigNumber>;
        exists(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
        freeMintSigner(overrides?: CallOverrides): Promise<string>;
        getApproved(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        getNextDerivedKeyId(overrides?: CallOverrides): Promise<string>;
        getPkpNftMetadataAddress(overrides?: CallOverrides): Promise<string>;
        getPkpPermissionsAddress(overrides?: CallOverrides): Promise<string>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        getRouterAddress(overrides?: CallOverrides): Promise<string>;
        getStakingAddress(overrides?: CallOverrides): Promise<string>;
        initialize(overrides?: CallOverrides): Promise<void>;
        isApprovedForAll(owner: PromiseOrValue<string>, operator: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
        mintCost(overrides?: CallOverrides): Promise<BigNumber>;
        mintGrantAndBurnNext(keyType: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        mintNext(keyType: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        name(overrides?: CallOverrides): Promise<string>;
        ownerOf(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        prefixed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
        redeemedFreeMintIds(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
        "safeTransferFrom(address,address,uint256)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        "safeTransferFrom(address,address,uint256,bytes)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, data: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        setApprovalForAll(operator: PromiseOrValue<string>, approved: PromiseOrValue<boolean>, overrides?: CallOverrides): Promise<void>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        setFreeMintSigner(newFreeMintSigner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        setMintCost(newMintCost: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        supportsInterface(interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        symbol(overrides?: CallOverrides): Promise<string>;
        tokenByIndex(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        tokenOfOwnerByIndex(owner: PromiseOrValue<string>, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        tokenURI(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        totalSupply(overrides?: CallOverrides): Promise<BigNumber>;
        transferFrom(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        withdraw(overrides?: CallOverrides): Promise<void>;
    };
    filters: {
        "DiamondCut(tuple[],address,bytes)"(_diamondCut?: null, _init?: null, _calldata?: null): DiamondCutEventFilter;
        DiamondCut(_diamondCut?: null, _init?: null, _calldata?: null): DiamondCutEventFilter;
        "OwnershipTransferred(address,address)"(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        OwnershipTransferred(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        "Approval(address,address,uint256)"(owner?: PromiseOrValue<string> | null, approved?: PromiseOrValue<string> | null, tokenId?: PromiseOrValue<BigNumberish> | null): ApprovalEventFilter;
        Approval(owner?: PromiseOrValue<string> | null, approved?: PromiseOrValue<string> | null, tokenId?: PromiseOrValue<BigNumberish> | null): ApprovalEventFilter;
        "ApprovalForAll(address,address,bool)"(owner?: PromiseOrValue<string> | null, operator?: PromiseOrValue<string> | null, approved?: null): ApprovalForAllEventFilter;
        ApprovalForAll(owner?: PromiseOrValue<string> | null, operator?: PromiseOrValue<string> | null, approved?: null): ApprovalForAllEventFilter;
        "ContractResolverAddressSet(address)"(newResolverAddress?: null): ContractResolverAddressSetEventFilter;
        ContractResolverAddressSet(newResolverAddress?: null): ContractResolverAddressSetEventFilter;
        "FreeMintSignerSet(address)"(newFreeMintSigner?: PromiseOrValue<string> | null): FreeMintSignerSetEventFilter;
        FreeMintSignerSet(newFreeMintSigner?: PromiseOrValue<string> | null): FreeMintSignerSetEventFilter;
        "Initialized(uint8)"(version?: null): InitializedEventFilter;
        Initialized(version?: null): InitializedEventFilter;
        "MintCostSet(uint256)"(newMintCost?: null): MintCostSetEventFilter;
        MintCostSet(newMintCost?: null): MintCostSetEventFilter;
        "PKPMinted(uint256,bytes)"(tokenId?: PromiseOrValue<BigNumberish> | null, pubkey?: null): PKPMintedEventFilter;
        PKPMinted(tokenId?: PromiseOrValue<BigNumberish> | null, pubkey?: null): PKPMintedEventFilter;
        "Transfer(address,address,uint256)"(from?: PromiseOrValue<string> | null, to?: PromiseOrValue<string> | null, tokenId?: PromiseOrValue<BigNumberish> | null): TransferEventFilter;
        Transfer(from?: PromiseOrValue<string> | null, to?: PromiseOrValue<string> | null, tokenId?: PromiseOrValue<BigNumberish> | null): TransferEventFilter;
        "Withdrew(uint256)"(amount?: null): WithdrewEventFilter;
        Withdrew(amount?: null): WithdrewEventFilter;
    };
    estimateGas: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        facetAddresses(overrides?: CallOverrides): Promise<BigNumber>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        facets(overrides?: CallOverrides): Promise<BigNumber>;
        owner(overrides?: CallOverrides): Promise<BigNumber>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        approve(to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        balanceOf(owner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        burn(tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        claimAndMint(keyType: PromiseOrValue<BigNumberish>, derivedKeyId: PromiseOrValue<BytesLike>, signatures: IPubkeyRouter.SignatureStruct[], overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        exists(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        freeMintSigner(overrides?: CallOverrides): Promise<BigNumber>;
        getApproved(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getNextDerivedKeyId(overrides?: CallOverrides): Promise<BigNumber>;
        getPkpNftMetadataAddress(overrides?: CallOverrides): Promise<BigNumber>;
        getPkpPermissionsAddress(overrides?: CallOverrides): Promise<BigNumber>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getRouterAddress(overrides?: CallOverrides): Promise<BigNumber>;
        getStakingAddress(overrides?: CallOverrides): Promise<BigNumber>;
        initialize(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        isApprovedForAll(owner: PromiseOrValue<string>, operator: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        mintCost(overrides?: CallOverrides): Promise<BigNumber>;
        mintGrantAndBurnNext(keyType: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        mintNext(keyType: PromiseOrValue<BigNumberish>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        name(overrides?: CallOverrides): Promise<BigNumber>;
        ownerOf(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        prefixed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        redeemedFreeMintIds(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        "safeTransferFrom(address,address,uint256)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        "safeTransferFrom(address,address,uint256,bytes)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, data: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setApprovalForAll(operator: PromiseOrValue<string>, approved: PromiseOrValue<boolean>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setFreeMintSigner(newFreeMintSigner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setMintCost(newMintCost: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        supportsInterface(interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        symbol(overrides?: CallOverrides): Promise<BigNumber>;
        tokenByIndex(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        tokenOfOwnerByIndex(owner: PromiseOrValue<string>, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        tokenURI(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        totalSupply(overrides?: CallOverrides): Promise<BigNumber>;
        transferFrom(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        withdraw(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
    };
    populateTransaction: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facetAddresses(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facets(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        owner(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        approve(to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        balanceOf(owner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        burn(tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        claimAndMint(keyType: PromiseOrValue<BigNumberish>, derivedKeyId: PromiseOrValue<BytesLike>, signatures: IPubkeyRouter.SignatureStruct[], overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        exists(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        freeMintSigner(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getApproved(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getNextDerivedKeyId(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPkpNftMetadataAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPkpPermissionsAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getRouterAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getStakingAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        initialize(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        isApprovedForAll(owner: PromiseOrValue<string>, operator: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        mintCost(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        mintGrantAndBurnNext(keyType: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        mintNext(keyType: PromiseOrValue<BigNumberish>, overrides?: PayableOverrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        name(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        ownerOf(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        prefixed(hash: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        redeemedFreeMintIds(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        "safeTransferFrom(address,address,uint256)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        "safeTransferFrom(address,address,uint256,bytes)"(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, data: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setApprovalForAll(operator: PromiseOrValue<string>, approved: PromiseOrValue<boolean>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setFreeMintSigner(newFreeMintSigner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setMintCost(newMintCost: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        supportsInterface(interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        symbol(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        tokenByIndex(index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        tokenOfOwnerByIndex(owner: PromiseOrValue<string>, index: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        tokenURI(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        totalSupply(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        transferFrom(from: PromiseOrValue<string>, to: PromiseOrValue<string>, tokenId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        withdraw(overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
    };
}

export interface TypedEvent<TArgsArray extends Array<any> = any, TArgsObject = any> extends Event {
    args: TArgsArray & TArgsObject;
}
export interface TypedEventFilter<_TEvent extends TypedEvent> extends EventFilter {
}
export interface TypedListener<TEvent extends TypedEvent> {
    (...listenerArg: [...__TypechainArgsArray<TEvent>, TEvent]): void;
}
type __TypechainArgsArray<T> = T extends TypedEvent<infer U> ? U : never;
export interface OnEvent<TRes> {
    <TEvent extends TypedEvent>(eventFilter: TypedEventFilter<TEvent>, listener: TypedListener<TEvent>): TRes;
    (eventName: string, listener: Listener): TRes;
}
export type MinEthersFactory<C, ARGS> = {
    deploy(...a: ARGS[]): Promise<C>;
};
export type GetContractTypeFromFactory<F> = F extends MinEthersFactory<infer C, any> ? C : never;
export type GetARGsTypeFromFactory<F> = F extends MinEthersFactory<any, any> ? Parameters<F["deploy"]> : never;
export type PromiseOrValue<T> = T | Promise<T>;

export declare namespace IDiamond {
    type FacetCutStruct = {
        facetAddress: PromiseOrValue<string>;
        action: PromiseOrValue<BigNumberish>;
        functionSelectors: PromiseOrValue<BytesLike>[];
    };
    type FacetCutStructOutput = [string, number, string[]] & {
        facetAddress: string;
        action: number;
        functionSelectors: string[];
    };
}
export declare namespace IDiamondLoupe {
    type FacetStruct = {
        facetAddress: PromiseOrValue<string>;
        functionSelectors: PromiseOrValue<BytesLike>[];
    };
    type FacetStructOutput = [string, string[]] & {
        facetAddress: string;
        functionSelectors: string[];
    };
}
export declare namespace LibPKPPermissionsStorage {
    type AuthMethodStruct = {
        authMethodType: PromiseOrValue<BigNumberish>;
        id: PromiseOrValue<BytesLike>;
        userPubkey: PromiseOrValue<BytesLike>;
    };
    type AuthMethodStructOutput = [BigNumber, string, string] & {
        authMethodType: BigNumber;
        id: string;
        userPubkey: string;
    };
}
export interface PKPPermissionsInterface extends utils.Interface {
    functions: {
        "diamondCut((address,uint8,bytes4[])[],address,bytes)": FunctionFragment;
        "facetAddress(bytes4)": FunctionFragment;
        "facetAddresses()": FunctionFragment;
        "facetFunctionSelectors(address)": FunctionFragment;
        "facets()": FunctionFragment;
        "supportsInterface(bytes4)": FunctionFragment;
        "owner()": FunctionFragment;
        "transferOwnership(address)": FunctionFragment;
        "addPermittedAction(uint256,bytes,uint256[])": FunctionFragment;
        "addPermittedAddress(uint256,address,uint256[])": FunctionFragment;
        "addPermittedAuthMethod(uint256,(uint256,bytes,bytes),uint256[])": FunctionFragment;
        "addPermittedAuthMethodScope(uint256,uint256,bytes,uint256)": FunctionFragment;
        "batchAddRemoveAuthMethods(uint256,uint256[],bytes[],bytes[],uint256[][],uint256[],bytes[])": FunctionFragment;
        "getAuthMethodId(uint256,bytes)": FunctionFragment;
        "getEthAddress(uint256)": FunctionFragment;
        "getPermittedActions(uint256)": FunctionFragment;
        "getPermittedAddresses(uint256)": FunctionFragment;
        "getPermittedAuthMethodScopes(uint256,uint256,bytes,uint256)": FunctionFragment;
        "getPermittedAuthMethods(uint256)": FunctionFragment;
        "getPkpNftAddress()": FunctionFragment;
        "getPubkey(uint256)": FunctionFragment;
        "getRouterAddress()": FunctionFragment;
        "getTokenIdsForAuthMethod(uint256,bytes)": FunctionFragment;
        "getUserPubkeyForAuthMethod(uint256,bytes)": FunctionFragment;
        "isPermittedAction(uint256,bytes)": FunctionFragment;
        "isPermittedAddress(uint256,address)": FunctionFragment;
        "isPermittedAuthMethod(uint256,uint256,bytes)": FunctionFragment;
        "isPermittedAuthMethodScopePresent(uint256,uint256,bytes,uint256)": FunctionFragment;
        "removePermittedAction(uint256,bytes)": FunctionFragment;
        "removePermittedAddress(uint256,address)": FunctionFragment;
        "removePermittedAuthMethod(uint256,uint256,bytes)": FunctionFragment;
        "removePermittedAuthMethodScope(uint256,uint256,bytes,uint256)": FunctionFragment;
        "setContractResolver(address)": FunctionFragment;
        "setRootHash(uint256,uint256,bytes32)": FunctionFragment;
        "verifyState(uint256,uint256,bytes32[],bytes32)": FunctionFragment;
        "verifyStates(uint256,uint256,bytes32[],bool[],bytes32[])": FunctionFragment;
    };
    getFunction(nameOrSignatureOrTopic: "diamondCut" | "facetAddress" | "facetAddresses" | "facetFunctionSelectors" | "facets" | "supportsInterface" | "owner" | "transferOwnership" | "addPermittedAction" | "addPermittedAddress" | "addPermittedAuthMethod" | "addPermittedAuthMethodScope" | "batchAddRemoveAuthMethods" | "getAuthMethodId" | "getEthAddress" | "getPermittedActions" | "getPermittedAddresses" | "getPermittedAuthMethodScopes" | "getPermittedAuthMethods" | "getPkpNftAddress" | "getPubkey" | "getRouterAddress" | "getTokenIdsForAuthMethod" | "getUserPubkeyForAuthMethod" | "isPermittedAction" | "isPermittedAddress" | "isPermittedAuthMethod" | "isPermittedAuthMethodScopePresent" | "removePermittedAction" | "removePermittedAddress" | "removePermittedAuthMethod" | "removePermittedAuthMethodScope" | "setContractResolver" | "setRootHash" | "verifyState" | "verifyStates"): FunctionFragment;
    encodeFunctionData(functionFragment: "diamondCut", values: [
        IDiamond.FacetCutStruct[],
        PromiseOrValue<string>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "facetAddress", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "facetAddresses", values?: undefined): string;
    encodeFunctionData(functionFragment: "facetFunctionSelectors", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "facets", values?: undefined): string;
    encodeFunctionData(functionFragment: "supportsInterface", values: [PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "owner", values?: undefined): string;
    encodeFunctionData(functionFragment: "transferOwnership", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "addPermittedAction", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        PromiseOrValue<BigNumberish>[]
    ]): string;
    encodeFunctionData(functionFragment: "addPermittedAddress", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<string>,
        PromiseOrValue<BigNumberish>[]
    ]): string;
    encodeFunctionData(functionFragment: "addPermittedAuthMethod", values: [
        PromiseOrValue<BigNumberish>,
        LibPKPPermissionsStorage.AuthMethodStruct,
        PromiseOrValue<BigNumberish>[]
    ]): string;
    encodeFunctionData(functionFragment: "addPermittedAuthMethodScope", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "batchAddRemoveAuthMethods", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>[],
        PromiseOrValue<BytesLike>[],
        PromiseOrValue<BytesLike>[],
        PromiseOrValue<BigNumberish>[][],
        PromiseOrValue<BigNumberish>[],
        PromiseOrValue<BytesLike>[]
    ]): string;
    encodeFunctionData(functionFragment: "getAuthMethodId", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "getEthAddress", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getPermittedActions", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getPermittedAddresses", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getPermittedAuthMethodScopes", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "getPermittedAuthMethods", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getPkpNftAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "getPubkey", values: [PromiseOrValue<BigNumberish>]): string;
    encodeFunctionData(functionFragment: "getRouterAddress", values?: undefined): string;
    encodeFunctionData(functionFragment: "getTokenIdsForAuthMethod", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "getUserPubkeyForAuthMethod", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "isPermittedAction", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "isPermittedAddress", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "isPermittedAuthMethod", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "isPermittedAuthMethodScopePresent", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "removePermittedAction", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<BytesLike>]): string;
    encodeFunctionData(functionFragment: "removePermittedAddress", values: [PromiseOrValue<BigNumberish>, PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "removePermittedAuthMethod", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "removePermittedAuthMethodScope", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>,
        PromiseOrValue<BigNumberish>
    ]): string;
    encodeFunctionData(functionFragment: "setContractResolver", values: [PromiseOrValue<string>]): string;
    encodeFunctionData(functionFragment: "setRootHash", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "verifyState", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>[],
        PromiseOrValue<BytesLike>
    ]): string;
    encodeFunctionData(functionFragment: "verifyStates", values: [
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BigNumberish>,
        PromiseOrValue<BytesLike>[],
        PromiseOrValue<boolean>[],
        PromiseOrValue<BytesLike>[]
    ]): string;
    decodeFunctionResult(functionFragment: "diamondCut", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetAddresses", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facetFunctionSelectors", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "facets", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "supportsInterface", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "owner", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "transferOwnership", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "addPermittedAction", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "addPermittedAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "addPermittedAuthMethod", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "addPermittedAuthMethodScope", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "batchAddRemoveAuthMethods", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getAuthMethodId", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getEthAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPermittedActions", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPermittedAddresses", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPermittedAuthMethodScopes", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPermittedAuthMethods", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPkpNftAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getPubkey", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getRouterAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getTokenIdsForAuthMethod", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "getUserPubkeyForAuthMethod", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isPermittedAction", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isPermittedAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isPermittedAuthMethod", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "isPermittedAuthMethodScopePresent", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "removePermittedAction", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "removePermittedAddress", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "removePermittedAuthMethod", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "removePermittedAuthMethodScope", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setContractResolver", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "setRootHash", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "verifyState", data: BytesLike): Result;
    decodeFunctionResult(functionFragment: "verifyStates", data: BytesLike): Result;
    events: {
        "DiamondCut(tuple[],address,bytes)": EventFragment;
        "OwnershipTransferred(address,address)": EventFragment;
        "ContractResolverAddressSet(address)": EventFragment;
        "PermittedAuthMethodAdded(uint256,uint256,bytes,bytes)": EventFragment;
        "PermittedAuthMethodRemoved(uint256,uint256,bytes)": EventFragment;
        "PermittedAuthMethodScopeAdded(uint256,uint256,bytes,uint256)": EventFragment;
        "PermittedAuthMethodScopeRemoved(uint256,uint256,bytes,uint256)": EventFragment;
        "RootHashUpdated(uint256,uint256,bytes32)": EventFragment;
    };
    getEvent(nameOrSignatureOrTopic: "DiamondCut"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "OwnershipTransferred"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "ContractResolverAddressSet"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "PermittedAuthMethodAdded"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "PermittedAuthMethodRemoved"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "PermittedAuthMethodScopeAdded"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "PermittedAuthMethodScopeRemoved"): EventFragment;
    getEvent(nameOrSignatureOrTopic: "RootHashUpdated"): EventFragment;
}
export interface DiamondCutEventObject {
    _diamondCut: IDiamond.FacetCutStructOutput[];
    _init: string;
    _calldata: string;
}
export type DiamondCutEvent = TypedEvent<[
    IDiamond.FacetCutStructOutput[],
    string,
    string
], DiamondCutEventObject>;
export type DiamondCutEventFilter = TypedEventFilter<DiamondCutEvent>;
export interface OwnershipTransferredEventObject {
    previousOwner: string;
    newOwner: string;
}
export type OwnershipTransferredEvent = TypedEvent<[
    string,
    string
], OwnershipTransferredEventObject>;
export type OwnershipTransferredEventFilter = TypedEventFilter<OwnershipTransferredEvent>;
export interface ContractResolverAddressSetEventObject {
    newResolverAddress: string;
}
export type ContractResolverAddressSetEvent = TypedEvent<[
    string
], ContractResolverAddressSetEventObject>;
export type ContractResolverAddressSetEventFilter = TypedEventFilter<ContractResolverAddressSetEvent>;
export interface PermittedAuthMethodAddedEventObject {
    tokenId: BigNumber;
    authMethodType: BigNumber;
    id: string;
    userPubkey: string;
}
export type PermittedAuthMethodAddedEvent = TypedEvent<[
    BigNumber,
    BigNumber,
    string,
    string
], PermittedAuthMethodAddedEventObject>;
export type PermittedAuthMethodAddedEventFilter = TypedEventFilter<PermittedAuthMethodAddedEvent>;
export interface PermittedAuthMethodRemovedEventObject {
    tokenId: BigNumber;
    authMethodType: BigNumber;
    id: string;
}
export type PermittedAuthMethodRemovedEvent = TypedEvent<[
    BigNumber,
    BigNumber,
    string
], PermittedAuthMethodRemovedEventObject>;
export type PermittedAuthMethodRemovedEventFilter = TypedEventFilter<PermittedAuthMethodRemovedEvent>;
export interface PermittedAuthMethodScopeAddedEventObject {
    tokenId: BigNumber;
    authMethodType: BigNumber;
    id: string;
    scopeId: BigNumber;
}
export type PermittedAuthMethodScopeAddedEvent = TypedEvent<[
    BigNumber,
    BigNumber,
    string,
    BigNumber
], PermittedAuthMethodScopeAddedEventObject>;
export type PermittedAuthMethodScopeAddedEventFilter = TypedEventFilter<PermittedAuthMethodScopeAddedEvent>;
export interface PermittedAuthMethodScopeRemovedEventObject {
    tokenId: BigNumber;
    authMethodType: BigNumber;
    id: string;
    scopeId: BigNumber;
}
export type PermittedAuthMethodScopeRemovedEvent = TypedEvent<[
    BigNumber,
    BigNumber,
    string,
    BigNumber
], PermittedAuthMethodScopeRemovedEventObject>;
export type PermittedAuthMethodScopeRemovedEventFilter = TypedEventFilter<PermittedAuthMethodScopeRemovedEvent>;
export interface RootHashUpdatedEventObject {
    tokenId: BigNumber;
    group: BigNumber;
    root: string;
}
export type RootHashUpdatedEvent = TypedEvent<[
    BigNumber,
    BigNumber,
    string
], RootHashUpdatedEventObject>;
export type RootHashUpdatedEventFilter = TypedEventFilter<RootHashUpdatedEvent>;
export interface PKPPermissions extends BaseContract {
    connect(signerOrProvider: Signer | Provider | string): this;
    attach(addressOrName: string): this;
    deployed(): Promise<this>;
    interface: PKPPermissionsInterface;
    queryFilter<TEvent extends TypedEvent>(event: TypedEventFilter<TEvent>, fromBlockOrBlockhash?: string | number | undefined, toBlock?: string | number | undefined): Promise<Array<TEvent>>;
    listeners<TEvent extends TypedEvent>(eventFilter?: TypedEventFilter<TEvent>): Array<TypedListener<TEvent>>;
    listeners(eventName?: string): Array<Listener>;
    removeAllListeners<TEvent extends TypedEvent>(eventFilter: TypedEventFilter<TEvent>): this;
    removeAllListeners(eventName?: string): this;
    off: OnEvent<this>;
    on: OnEvent<this>;
    once: OnEvent<this>;
    removeListener: OnEvent<this>;
    functions: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[string] & {
            facetAddress_: string;
        }>;
        facetAddresses(overrides?: CallOverrides): Promise<[string[]] & {
            facetAddresses_: string[];
        }>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[string[]] & {
            _facetFunctionSelectors: string[];
        }>;
        facets(overrides?: CallOverrides): Promise<[
            IDiamondLoupe.FacetStructOutput[]
        ] & {
            facets_: IDiamondLoupe.FacetStructOutput[];
        }>;
        supportsInterface(_interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        owner(overrides?: CallOverrides): Promise<[string] & {
            owner_: string;
        }>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        addPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        addPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        addPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethod: LibPKPPermissionsStorage.AuthMethodStruct, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        addPermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        batchAddRemoveAuthMethods(tokenId: PromiseOrValue<BigNumberish>, permittedAuthMethodTypesToAdd: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodPubkeysToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodScopesToAdd: PromiseOrValue<BigNumberish>[][], permittedAuthMethodTypesToRemove: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToRemove: PromiseOrValue<BytesLike>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        getAuthMethodId(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[BigNumber]>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        getPermittedActions(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string[]]>;
        getPermittedAddresses(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string[]]>;
        getPermittedAuthMethodScopes(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, maxScopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[boolean[]]>;
        getPermittedAuthMethods(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[LibPKPPermissionsStorage.AuthMethodStructOutput[]]>;
        getPkpNftAddress(overrides?: CallOverrides): Promise<[string]>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[string]>;
        getRouterAddress(overrides?: CallOverrides): Promise<[string]>;
        getTokenIdsForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[BigNumber[]]>;
        getUserPubkeyForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[string]>;
        isPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        isPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<[boolean]>;
        isPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        isPermittedAuthMethodScopePresent(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<[boolean]>;
        removePermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        removePermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        removePermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        removePermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        setRootHash(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, root: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<ContractTransaction>;
        verifyState(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], leaf: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<[boolean]>;
        verifyStates(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], proofFlags: PromiseOrValue<boolean>[], leaves: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<[boolean]>;
    };
    diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
    facetAddresses(overrides?: CallOverrides): Promise<string[]>;
    facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<string[]>;
    facets(overrides?: CallOverrides): Promise<IDiamondLoupe.FacetStructOutput[]>;
    supportsInterface(_interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    owner(overrides?: CallOverrides): Promise<string>;
    transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    addPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    addPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    addPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethod: LibPKPPermissionsStorage.AuthMethodStruct, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    addPermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    batchAddRemoveAuthMethods(tokenId: PromiseOrValue<BigNumberish>, permittedAuthMethodTypesToAdd: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodPubkeysToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodScopesToAdd: PromiseOrValue<BigNumberish>[][], permittedAuthMethodTypesToRemove: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToRemove: PromiseOrValue<BytesLike>[], overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    getAuthMethodId(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
    getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    getPermittedActions(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string[]>;
    getPermittedAddresses(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string[]>;
    getPermittedAuthMethodScopes(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, maxScopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean[]>;
    getPermittedAuthMethods(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<LibPKPPermissionsStorage.AuthMethodStructOutput[]>;
    getPkpNftAddress(overrides?: CallOverrides): Promise<string>;
    getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
    getRouterAddress(overrides?: CallOverrides): Promise<string>;
    getTokenIdsForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber[]>;
    getUserPubkeyForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
    isPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    isPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
    isPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    isPermittedAuthMethodScopePresent(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
    removePermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    removePermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    removePermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    removePermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    setRootHash(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, root: PromiseOrValue<BytesLike>, overrides?: Overrides & {
        from?: PromiseOrValue<string>;
    }): Promise<ContractTransaction>;
    verifyState(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], leaf: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
    verifyStates(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], proofFlags: PromiseOrValue<boolean>[], leaves: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<boolean>;
    callStatic: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
        facetAddresses(overrides?: CallOverrides): Promise<string[]>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<string[]>;
        facets(overrides?: CallOverrides): Promise<IDiamondLoupe.FacetStructOutput[]>;
        supportsInterface(_interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        owner(overrides?: CallOverrides): Promise<string>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        addPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, scopes: PromiseOrValue<BigNumberish>[], overrides?: CallOverrides): Promise<void>;
        addPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, scopes: PromiseOrValue<BigNumberish>[], overrides?: CallOverrides): Promise<void>;
        addPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethod: LibPKPPermissionsStorage.AuthMethodStruct, scopes: PromiseOrValue<BigNumberish>[], overrides?: CallOverrides): Promise<void>;
        addPermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        batchAddRemoveAuthMethods(tokenId: PromiseOrValue<BigNumberish>, permittedAuthMethodTypesToAdd: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodPubkeysToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodScopesToAdd: PromiseOrValue<BigNumberish>[][], permittedAuthMethodTypesToRemove: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToRemove: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<void>;
        getAuthMethodId(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        getPermittedActions(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string[]>;
        getPermittedAddresses(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string[]>;
        getPermittedAuthMethodScopes(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, maxScopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean[]>;
        getPermittedAuthMethods(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<LibPKPPermissionsStorage.AuthMethodStructOutput[]>;
        getPkpNftAddress(overrides?: CallOverrides): Promise<string>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<string>;
        getRouterAddress(overrides?: CallOverrides): Promise<string>;
        getTokenIdsForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber[]>;
        getUserPubkeyForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<string>;
        isPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        isPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<boolean>;
        isPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        isPermittedAuthMethodScopePresent(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<boolean>;
        removePermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        removePermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        removePermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        removePermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<void>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: CallOverrides): Promise<void>;
        setRootHash(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, root: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<void>;
        verifyState(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], leaf: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<boolean>;
        verifyStates(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], proofFlags: PromiseOrValue<boolean>[], leaves: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<boolean>;
    };
    filters: {
        "DiamondCut(tuple[],address,bytes)"(_diamondCut?: null, _init?: null, _calldata?: null): DiamondCutEventFilter;
        DiamondCut(_diamondCut?: null, _init?: null, _calldata?: null): DiamondCutEventFilter;
        "OwnershipTransferred(address,address)"(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        OwnershipTransferred(previousOwner?: PromiseOrValue<string> | null, newOwner?: PromiseOrValue<string> | null): OwnershipTransferredEventFilter;
        "ContractResolverAddressSet(address)"(newResolverAddress?: null): ContractResolverAddressSetEventFilter;
        ContractResolverAddressSet(newResolverAddress?: null): ContractResolverAddressSetEventFilter;
        "PermittedAuthMethodAdded(uint256,uint256,bytes,bytes)"(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, userPubkey?: null): PermittedAuthMethodAddedEventFilter;
        PermittedAuthMethodAdded(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, userPubkey?: null): PermittedAuthMethodAddedEventFilter;
        "PermittedAuthMethodRemoved(uint256,uint256,bytes)"(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null): PermittedAuthMethodRemovedEventFilter;
        PermittedAuthMethodRemoved(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null): PermittedAuthMethodRemovedEventFilter;
        "PermittedAuthMethodScopeAdded(uint256,uint256,bytes,uint256)"(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, scopeId?: null): PermittedAuthMethodScopeAddedEventFilter;
        PermittedAuthMethodScopeAdded(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, scopeId?: null): PermittedAuthMethodScopeAddedEventFilter;
        "PermittedAuthMethodScopeRemoved(uint256,uint256,bytes,uint256)"(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, scopeId?: null): PermittedAuthMethodScopeRemovedEventFilter;
        PermittedAuthMethodScopeRemoved(tokenId?: PromiseOrValue<BigNumberish> | null, authMethodType?: null, id?: null, scopeId?: null): PermittedAuthMethodScopeRemovedEventFilter;
        "RootHashUpdated(uint256,uint256,bytes32)"(tokenId?: PromiseOrValue<BigNumberish> | null, group?: PromiseOrValue<BigNumberish> | null, root?: null): RootHashUpdatedEventFilter;
        RootHashUpdated(tokenId?: PromiseOrValue<BigNumberish> | null, group?: PromiseOrValue<BigNumberish> | null, root?: null): RootHashUpdatedEventFilter;
    };
    estimateGas: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        facetAddresses(overrides?: CallOverrides): Promise<BigNumber>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        facets(overrides?: CallOverrides): Promise<BigNumber>;
        supportsInterface(_interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        owner(overrides?: CallOverrides): Promise<BigNumber>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        addPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        addPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        addPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethod: LibPKPPermissionsStorage.AuthMethodStruct, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        addPermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        batchAddRemoveAuthMethods(tokenId: PromiseOrValue<BigNumberish>, permittedAuthMethodTypesToAdd: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodPubkeysToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodScopesToAdd: PromiseOrValue<BigNumberish>[][], permittedAuthMethodTypesToRemove: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToRemove: PromiseOrValue<BytesLike>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        getAuthMethodId(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getPermittedActions(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getPermittedAddresses(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getPermittedAuthMethodScopes(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, maxScopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getPermittedAuthMethods(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getPkpNftAddress(overrides?: CallOverrides): Promise<BigNumber>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        getRouterAddress(overrides?: CallOverrides): Promise<BigNumber>;
        getTokenIdsForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        getUserPubkeyForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        isPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        isPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<BigNumber>;
        isPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        isPermittedAuthMethodScopePresent(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<BigNumber>;
        removePermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        removePermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        removePermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        removePermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        setRootHash(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, root: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<BigNumber>;
        verifyState(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], leaf: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<BigNumber>;
        verifyStates(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], proofFlags: PromiseOrValue<boolean>[], leaves: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<BigNumber>;
    };
    populateTransaction: {
        diamondCut(_diamondCut: IDiamond.FacetCutStruct[], _init: PromiseOrValue<string>, _calldata: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        facetAddress(_functionSelector: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facetAddresses(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facetFunctionSelectors(_facet: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        facets(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        supportsInterface(_interfaceId: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        owner(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        transferOwnership(_newOwner: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        addPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        addPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        addPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethod: LibPKPPermissionsStorage.AuthMethodStruct, scopes: PromiseOrValue<BigNumberish>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        addPermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        batchAddRemoveAuthMethods(tokenId: PromiseOrValue<BigNumberish>, permittedAuthMethodTypesToAdd: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodPubkeysToAdd: PromiseOrValue<BytesLike>[], permittedAuthMethodScopesToAdd: PromiseOrValue<BigNumberish>[][], permittedAuthMethodTypesToRemove: PromiseOrValue<BigNumberish>[], permittedAuthMethodIdsToRemove: PromiseOrValue<BytesLike>[], overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        getAuthMethodId(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getEthAddress(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPermittedActions(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPermittedAddresses(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPermittedAuthMethodScopes(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, maxScopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPermittedAuthMethods(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPkpNftAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getPubkey(tokenId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getRouterAddress(overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getTokenIdsForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        getUserPubkeyForAuthMethod(authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isPermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isPermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isPermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        isPermittedAuthMethodScopePresent(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        removePermittedAction(tokenId: PromiseOrValue<BigNumberish>, ipfsCID: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        removePermittedAddress(tokenId: PromiseOrValue<BigNumberish>, user: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        removePermittedAuthMethod(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        removePermittedAuthMethodScope(tokenId: PromiseOrValue<BigNumberish>, authMethodType: PromiseOrValue<BigNumberish>, id: PromiseOrValue<BytesLike>, scopeId: PromiseOrValue<BigNumberish>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setContractResolver(newResolverAddress: PromiseOrValue<string>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        setRootHash(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, root: PromiseOrValue<BytesLike>, overrides?: Overrides & {
            from?: PromiseOrValue<string>;
        }): Promise<PopulatedTransaction>;
        verifyState(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], leaf: PromiseOrValue<BytesLike>, overrides?: CallOverrides): Promise<PopulatedTransaction>;
        verifyStates(tokenId: PromiseOrValue<BigNumberish>, group: PromiseOrValue<BigNumberish>, proof: PromiseOrValue<BytesLike>[], proofFlags: PromiseOrValue<boolean>[], leaves: PromiseOrValue<BytesLike>[], overrides?: CallOverrides): Promise<PopulatedTransaction>;
    };
}

