"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const src_1 = require("./../src");
const hardhat_1 = require("hardhat");
const constants_1 = require("./../src/constants");
describe("Verify the Execution Constraints", () => {
    let newCircuit, publicKey;
    const customActions = [
        {
            type: "custom",
            priority: 0,
            code: " async () => {  }",
        },
        {
            type: "custom",
            priority: 1,
            code: "async () => { }",
        },
    ];
    const chronicleProvider = new hardhat_1.ethers.providers.JsonRpcProvider(constants_1.CHRONICLE_PROVIDER, 175177);
    beforeEach(async () => {
        newCircuit = new src_1.Circuit(new hardhat_1.ethers.Wallet(process.env.PRIVATE_KEY, chronicleProvider));
        const LitActionCode = await newCircuit.setActions(customActions);
        newCircuit.setConditions([
            new src_1.WebhookCondition("https://api.weather.gov", "/zones/forecast/MIZ018/forecast", "geometry.type", "Polygon", "===", undefined, async () => {
                console.log("matched");
            }, async () => {
                console.log("unmatched");
            }, (err) => console.error(err.message)),
        ]);
        newCircuit.setConditionalLogic({
            type: "EVERY",
            interval: 1000,
        });
        const ipfsCID = await newCircuit.getIPFSHash(LitActionCode.litActionCode);
        const pkpTokenData = await newCircuit.mintGrantBurnPKP(ipfsCID);
        publicKey = pkpTokenData.publicKey;
    });
    describe("Successful Execution Constraints", () => {
        it("Execute Only to the Max Condition Monitor", async () => {
            newCircuit.executionConstraints({
                conditionMonitorExecutions: 3,
            });
            await newCircuit.start({ publicKey });
            (0, chai_1.expect)(newCircuit["conditionExecutedCount"]).to.equal(3);
        });
        it("Execute Only After the Start Date", async () => {
            let actionSet = false;
            const LitActionCode = await newCircuit.setActions([
                {
                    type: "custom",
                    priority: 2,
                    code: `async () => { actionSet = true; }`,
                    args: {
                        actionSet,
                    },
                },
            ]);
            const ipfsCID = await newCircuit.getIPFSHash(LitActionCode.litActionCode);
            const pkpTokenData = await newCircuit.mintGrantBurnPKP(ipfsCID);
            const publicKey = pkpTokenData.publicKey;
            const futureDate = new Date();
            futureDate.setSeconds(futureDate.getSeconds() + 20);
            newCircuit.executionConstraints({
                startDate: futureDate,
            });
            await newCircuit.start({ publicKey });
            await new Promise((resolve) => setTimeout(resolve, 2000));
            (0, chai_1.expect)(actionSet).to.false;
        });
        it("Execute Only Before the End Date", async () => {
            let actionSet = false;
            const LitActionCode = await newCircuit.setActions([
                {
                    type: "custom",
                    priority: 2,
                    code: `async () => { actionSet = true; }`,
                    args: {
                        actionSet,
                    },
                },
            ]);
            const ipfsCID = await newCircuit.getIPFSHash(LitActionCode.litActionCode);
            const pkpTokenData = await newCircuit.mintGrantBurnPKP(ipfsCID);
            const publicKey = pkpTokenData.publicKey;
            const endDate = new Date();
            endDate.setSeconds(endDate.getSeconds() - 20);
            newCircuit.executionConstraints({
                endDate: endDate,
            });
            await newCircuit.start({ publicKey });
            await new Promise((resolve) => setTimeout(resolve, 2000));
            (0, chai_1.expect)(actionSet).to.false;
        });
        it("Execute Only to the Lit Action Completions", async () => {
            newCircuit.executionConstraints({
                maxLitActionCompletions: 3,
            });
            await newCircuit.start({ publicKey });
            (0, chai_1.expect)(newCircuit["litActionCompletionCount"]).to.equal(3);
        });
        it("Execute Only to the Max Lit Action Completions with Higher Max Executions", async () => {
            newCircuit.executionConstraints({
                conditionMonitorExecutions: 4,
                maxLitActionCompletions: 1,
            });
            await newCircuit.start({ publicKey });
            (0, chai_1.expect)(newCircuit["litActionCompletionCount"]).to.equal(1);
            (0, chai_1.expect)(newCircuit["conditionExecutedCount"]).to.equal(1);
        });
        it("Execute Only to the Max Condition Monitor with Higher Lit Action Completions", async () => {
            newCircuit.executionConstraints({
                conditionMonitorExecutions: 1,
                maxLitActionCompletions: 2,
            });
            await newCircuit.start({ publicKey });
            (0, chai_1.expect)(newCircuit["litActionCompletionCount"]).to.equal(1);
            (0, chai_1.expect)(newCircuit["conditionExecutedCount"]).to.equal(1);
        });
        it("Continuously Execute for No Constraints Provided", async () => {
            const startPromise = newCircuit.start({ publicKey });
            setTimeout(() => {
                newCircuit.interrupt();
            }, 20000);
            await startPromise;
            (0, chai_1.expect)(newCircuit["conditionExecutedCount"]).to.be.greaterThan(5);
        });
    });
});
